package com.yuyu.srwildentity.listener;

import com.yuyu.srwildentity.conditionCheck.ConditionCheck;
import com.yuyu.srwildentity.config.ConfigManager;
import com.yuyu.srwildentity.config.condition.EntityCondition;
import com.yuyu.srwildentity.config.condition.EntitySite;
import com.yuyu.srwildentity.config.condition.LevelRefresh;
import com.yuyu.srwildentity.config.condition.SpawnEntityType;
import com.yuyu.srwildentity.pojo.AreaRefresh;
import com.yuyu.srwildentity.pojo.BlackListArea;
import com.yuyu.srwildentity.pojo.PlayerRefreshinfo;
import io.lumine.xikage.mythicmobs.MythicMobs;
import io.lumine.xikage.mythicmobs.api.exceptions.InvalidMobTypeException;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Biome;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;

import java.util.*;

/**
 * @author 峰。
 * @version 1.0
 * @project SrWildEntityNew
 * @date 2024/8/3 19:35:39
 * @description 针对单个玩家的监听类，注意这里要在实体刷新的监听类中统一管理
 */
public class PlayerRefreshLintener {
    private String playerName;
    private ConfigManager configManager;
    private Random random;
    private PlayerRefreshinfo playerRefreshinfo;
    private Integer workId;
    private boolean flag = false;
    private MythicMobs mythicMobs;

    public PlayerRefreshLintener(String player, ConfigManager configManager) {
        this.playerRefreshinfo = new PlayerRefreshinfo(playerName,0,new ArrayList<>());
        this.playerName = player;
        this.configManager = configManager;
        this.random = new Random();
        this.mythicMobs = MythicMobs.inst();

    }

    /**
     * 定时任务，每隔一段时间在玩家附近刷新实体
     */
    public void timedRdfreshEneity(){
        if (EntityRefreshListener.flag) {
            if (!flag){
                return;
            }

            if (EntityRefreshListener.logBool){
                EntityRefreshListener.log.info(ChatColor.GOLD+playerName+"执行刷新任务");
            }

            int num = configManager.getNum();//每次执行时刷新的总数

                int attempts = 0;
                int sum = 0;//用于记录此次刷新生成的总数
                //获取玩家信息
                Player player = Bukkit.getPlayer(playerName);
                //玩家不在线
                if (player == null){
                    if (EntityRefreshListener.logBool){
                        EntityRefreshListener.log.info(ChatColor.RED + "Player " + playerName + " not found");
                    }
                    //返回之前取消定时任务
                    Bukkit.getScheduler().cancelTask(this.workId);
                    return;
                }
                Location playerlocation = player.getLocation();
                World world = playerlocation.getWorld();

                String worldName = world.getName();
                boolean pass = false;

                //如果黑名单的set集合里面包含有该世界的名字，则跳过
                if (configManager.getBlacklistWorldSet().contains(worldName)){
                    if (EntityRefreshListener.logBool){
                        EntityRefreshListener.log.info(ChatColor.RED + "黑名单世界 " + worldName+"\t"+playerName);
                    }
                    return;
                }

                if (configManager.getBlackListAreaMap().containsKey(worldName)){
                    List<BlackListArea> blackListAreas = configManager.getBlackListAreaMap().get(worldName);
                    for (BlackListArea blackListArea : blackListAreas){
                        int blockX = playerlocation.getBlockX();
                        int blockZ = playerlocation.getBlockZ();
                        if (blackListArea.getX1() >= blockX && blockX >= blackListArea.getX2()
                                && blackListArea.getY1() >= blockZ && blockZ >= blackListArea.getY2()){
                            //tag为循环标志，此处会直接结束tag标志处的循环
                            if (EntityRefreshListener.logBool){
                                EntityRefreshListener.log.info(ChatColor.RED + "黑名单区域 " + worldName+"\t"+playerName);
                            }
                            return;
                        }
                    }

                }

                //玩家危险度,用于刷新实体的等级
                int riskLevel = EntityRefreshListener.prRisk(player);
                if (riskLevel == 0){
                    //等于0不刷新怪物,直接跳过
                    if (EntityRefreshListener.logBool){
                        EntityRefreshListener.log.info(ChatColor.RED+playerName+"危险度为0");
                    }
                    return;
                }

                LevelRefresh refreshList = null;
                List<LevelRefresh> levelRefreshesList = configManager.getBiomeEntityRefreshSettings().getLevelRefreshesList();
                for (LevelRefresh levelRefresh : levelRefreshesList){
                    if (riskLevel > levelRefresh.getRiskMin() && riskLevel <= levelRefresh.getRiskMax()){
                        refreshList = levelRefresh;
                    }
                }

                if (refreshList == null){
                    if ( EntityRefreshListener.logBool) {
                        EntityRefreshListener.log.info(ChatColor.MAGIC + playerName + "所在群系" + "没有危险度为:" + riskLevel + "时适配的刷新列表");
                    }
                    return;
                }



                //获取玩家位置信息
                int blockZ = playerlocation.getBlockZ();
                int blockX = playerlocation.getBlockX();
                int blockY = playerlocation.getBlockY();
                Biome biome = world.getBiome(blockX, blockZ);
                String biomeName = biome.name();

                List<EntityCondition> entityConditionList = null;

                HashMap<String, List<EntityCondition>> entityConditionHashMap = refreshList.getEntityConditionHashMap();
                for (String refreshBiome : entityConditionHashMap.keySet()){
                    if (biomeName.contains(refreshBiome)){
                        entityConditionList = entityConditionHashMap.get(refreshBiome);
                    }
                }


                if (entityConditionList == null){
                    if (EntityRefreshListener.logBool) {
                        EntityRefreshListener.log.info(ChatColor.RED + "没有配置" + biomeName + "群系的实体刷新案例");
                    }
                    return;
                }

                for (EntityCondition entityCondition : entityConditionList){


                    int yMin = entityCondition.getyMin();
                    int yMax = entityCondition.getyMax();

                    //如果玩家所在位置不再设置的y轴高度内，则不刷新此entity
                    if (yMin > blockY || yMax < blockY){
                        if (EntityRefreshListener.logBool) {
                            EntityRefreshListener.log.info(ChatColor.RED + "刷新高度不通过" + entityCondition.getEntityName()+"\tyMax="+yMax+"\tyMin="+yMin+"\tblockY="+blockY);
                        }
                        continue;
                    }
                    float v = random.nextFloat();
                    if ( entityCondition.getWeight() <= v){
                        if (EntityRefreshListener.logBool) {
                            //刷新不通过,进入下一个验证的实体
                            EntityRefreshListener.log.info(ChatColor.RED + "刷新权重不通过" + entityCondition.getEntityName() + "权重:" + entityCondition.getWeight() + "\t" + v);
                        }
                        continue;
                    }

                    int nums = entityCondition.getNums();//当前实体需要刷新的数量

                    for (int i = 0;i<nums &&
                            playerRefreshinfo.getEntityList().size() < configManager.getTotal()
                            && sum <= num
                            && attempts <= configManager.getAttempts(); i++) {

                        // 生成x和y坐标,会随机在玩家方圆15个方块的距离内随机生成
                        int x =blockX + random.nextInt(30) - 15; // 生成0到60之间的随机数，然后减去30，得到-30到30的范围
                        int z =blockZ + random.nextInt(30) - 15; // 同上,在玩家附近随机位置生成
                        int y =blockY + random.nextInt(3) - 3; //在玩家Y轴上下3格尝试刷新
                        //刷新在地上，直接获取最高的x z 最高处的坐标
                        if (entityCondition.getEntitySite() == EntitySite.ON_GROUND) {

                            Location location = new Location(world, x, y , z);

                            //判断位置是否符合
                            if (ConditionCheck.checkEntityRefresh(world, location, entityCondition)) {
                                //通过则生成
//                                Entity entity = world.spawnEntity(location, EntityType.valueOf(entityCondition.getEntityName()));
                                Entity  entity;
                                //MC原生实体
                                if (entityCondition.getEntityType() == SpawnEntityType.PROTOGENESIS){
                                    entity = world.spawnEntity(location, EntityType.valueOf(entityCondition.getEntityName()));
                                }else {
                                    //刷新MM怪物
                                    try {
                                        entity = mythicMobs.getAPIHelper().spawnMythicMob(entityCondition.getEntityName(),location,riskLevel);
                                    } catch (InvalidMobTypeException e) {
                                        throw new RuntimeException(e);
                                    }
                                }

                                if (EntityRefreshListener.logBool) {
                                    EntityRefreshListener.log.info(ChatColor.GOLD + "在" + x + " " + y + " " + z + "位置刷新了" + entityCondition.getEntityName());
                                }
                                UUID uniqueId = entity.getUniqueId();

                                //存入玩家对应的刷新的实体集合，便于寻找
                                EntityRefreshListener.entityUUIDToPlayer.put(uniqueId, playerName);

                                //存入
                                playerRefreshinfo.addEntityList(uniqueId);
                                attempts++;
                                sum++;

                            }
                        } else {
                            //不刷新在地上，在定义的范围内随机高度，概率生成实体，
                            for (int c = 0; i<nums && c < 10 && sum <= num &&
                                    playerRefreshinfo.getEntityList().size() < configManager.getTotal();
                                 c++) {
                                //循环十次验证刷新位置
                                y = blockY + random.nextInt(3) - 3;//在玩家所在高度的上下三格内生成，便于当玩家在洞穴时，定位洞穴
                                x = blockX + random.nextInt(30) - 15; // 生成0到60之间的随机数，然后减去30，得到-30到30的范围
                                z = blockZ + random.nextInt(30) - 15; // 同上,在玩家附近随机位置生成
                                if (entityCondition.getEntitySite() == EntitySite.ON_GROUND) {
                                    y = world.getHighestBlockYAt(x,z);
                                }

                                Location location = new Location(world, x, y + 1, z);

                                //判断位置是否符合
                                if (ConditionCheck.checkEntityRefresh(world, location, entityCondition)) {
                                    //通过则生成
                                    Entity  entity;
                                    //MC原生实体
                                    if (entityCondition.getEntityType() == SpawnEntityType.PROTOGENESIS){
                                        entity = world.spawnEntity(location, EntityType.valueOf(entityCondition.getEntityName()));
                                    }else {
                                        //刷新MM怪物
                                        try {
                                            entity = MythicMobs.inst().getAPIHelper().spawnMythicMob(entityCondition.getEntityName(), location);
                                        } catch (InvalidMobTypeException e) {
                                            throw new RuntimeException(e);
                                        }
                                    }

                                    if (EntityRefreshListener.logBool) {
                                        EntityRefreshListener.log.info(ChatColor.GOLD + "在" + x + " " + y + " " + z + "位置刷新了" + entityCondition.getEntityName());
                                    }
                                    UUID uniqueId = entity.getUniqueId();

                                    //存入玩家对应的刷新的实体集合，便于寻找
                                    EntityRefreshListener.entityUUIDToPlayer.put(uniqueId, playerName);

                                    //存入
                                    playerRefreshinfo.addEntityList(uniqueId);
                                    i++;
                                    sum++;
                                    attempts++;
                                }
                            }
                        }
                    }
                }
        }
    }

    public Integer getWorkId() {
        return workId;
    }

    public void setWorkId(Integer workId) {
        this.workId = workId;
    }

    public ConfigManager getConfigManager() {
        return configManager;
    }

    public void setConfigManager(ConfigManager configManager) {
        this.configManager = configManager;
    }

    public PlayerRefreshinfo getPlayerRefreshinfo() {
        return playerRefreshinfo;
    }

    public void setPlayerRefreshinfo(PlayerRefreshinfo playerRefreshinfo) {
        this.playerRefreshinfo = playerRefreshinfo;
    }

    public boolean isFlag() {
        return flag;
    }

    public void setFlag(boolean flag) {
        this.flag = flag;
    }
}
